Class: Netzke::Basepack::DynamicTabPanel
- Inherits:
-
Netzke::Base
- Object
- Netzke::Base
- Netzke::Basepack::DynamicTabPanel
- Defined in:
- lib/netzke/basepack/dynamic_tab_panel.rb
Overview
A tab panel that can load components dynamically by their class name. Components can be loaded in the current or new tab. For example:
this.netzkeLoadComponentByClass('BookGrid', {newTab: true, serverConfig: {read_only: true}});